﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

using System.Numerics;
using System.Windows.Media.Imaging;

namespace SilverZoomer
{
    public partial class MainPage : UserControl
    {
        private Rect area = new Rect(new Point(-2.4, -1.5),new Point(0.8, 1.5));

        public MainPage()
        {
            InitializeComponent();
            canvas1.Children.Add(selection);

        }

        private WriteableBitmap drawSet(Rect area)
        {
            int PixelHeight = (int)image1.Height;
            int PixelWidth = (int)image1.Width;
            WriteableBitmap wbmap = new WriteableBitmap(PixelWidth, PixelHeight);
            
            double xscale = (area.Right - area.Left) / PixelWidth;
            double yscale = (area.Top - area.Bottom) / PixelHeight;
            int s = wbmap.PixelWidth;
            for (int i = 0; i < wbmap.Pixels.Length;i ++)
            {
                int py = i / s;
                int px = i % s;
                double x = area.Left + px * xscale;
                double y = area.Top - py * yscale;
                Complex c = new Complex(x, y);
                int count = mandelbrot(c);
                Color C = colorMap(count);
                wbmap.Pixels[i] = C.A << 24 | C.R << 16 | C.G << 8 | C.B;
              
            }
                return wbmap;
          }


        private Color colorMap(int count)
        {
            Color C = new Color();
            C.B = (byte)(count / 100 * 25);
            count = count % 100;
            C.G = (byte)(count / 10 * 25);
            C.R = (byte)(count % 10 * 25);
            C.A = 255;
            return C;
        }

        private Int32 mandelbrot(Complex c)
        {
            Int32 count = 0;
            Complex z = Complex.Zero;
            while (count < 1000 && z.Magnitude < 4)
            {
                z = z * z + c;
                count++;
            }
            return count;
        }


        private Rectangle selection =
             new Rectangle()
             {
                 Stroke = new SolidColorBrush(Colors.Black),
                 StrokeThickness = 1,
                 Visibility = Visibility.Collapsed
             };
        private bool mousedown = false;
        private Point mousedownpos;
        private void canvas1_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            mousedown = true;
            mousedownpos = e.GetPosition(canvas1);
            Canvas.SetLeft(selection, mousedownpos.X);
            Canvas.SetTop(selection, mousedownpos.Y);
            selection.Width = 0;
            selection.Height = 0;
            selection.Visibility = Visibility.Visible;
        }

        private void canvas1_MouseMove(object sender, MouseEventArgs e)
        {
            if (mousedown)
            {
                Point mousepos = e.GetPosition(canvas1);
                Point diff = new Point(mousepos.X - mousedownpos.X, mousepos.Y - mousedownpos.Y);
                Point TopLeft = mousedownpos;
                if (diff.X < 0)
                {
                    TopLeft.X = mousepos.X;
                    diff.X = -diff.X;
                }
                if (diff.Y < 0)
                {
                    TopLeft.Y = mousepos.Y;
                    diff.Y = -diff.Y;
                }
                selection.Width = diff.X;
                selection.Height = diff.Y;
                Canvas.SetLeft(selection, TopLeft.X);
                Canvas.SetTop(selection, TopLeft.Y);

            }

        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            area = new Rect(new Point(-2.4, -1.5), new Point(0.8, 1.5));
            image1.Source = drawSet(area);
        }

        private void canvas1_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            mousedown = false;
            selection.Visibility = Visibility.Collapsed;
            double xscale = (area.Right - area.Left) /image1.Width;
            double yscale = (area.Top - area.Bottom) / image1.Height;
            Point TopLeft = new Point(area.Left + Canvas.GetLeft(selection) * xscale,area.Top - Canvas.GetTop(selection) * yscale);
            Point BottomRight = new Point(TopLeft.X + selection.Width * xscale, TopLeft.Y -selection.Height * yscale);
            area = new Rect(TopLeft, BottomRight);
            image1.Source = drawSet(area);
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
           image1.Source = drawSet(area);
        }
    }
}
